﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;
using Ascon.Pilot.SDK;
using Ascon.Pilot.SDK.Automation;

namespace SignMeAsActivity
{
    [Export(typeof(IAutomationActivity))]
    public class SignMeAsActivity : AutomationActivity
    {
        private readonly IXpsDigitalSigner _digitalSigner;

        [ImportingConstructor]
        public SignMeAsActivity(IXpsDigitalSigner digitalSigner)
        {
            _digitalSigner = digitalSigner;
        }

        public override string Name => nameof(SignMeAsActivity);

        public override async Task RunAsync(IObjectModifier modifier, IAutomationBackend backend, IAutomationEventContext context, TriggerType triggerType)
        {
            var sourceObject = backend.GetObject(context.Source.Id);
            var person = context.InitiatingPerson;
            // check that my task
            if (sourceObject.Attributes.TryGetValue(SystemTaskAttributes.EXECUTOR_POSITION, out var executorAttribute))
            {
                if (executorAttribute is int[] executors)
                {
                    var executor = executors.FirstOrDefault(); 
                    if (person.Positions.FirstOrDefault(p => p.Position == executor) == null)
                        return;
                }
            }

            var attachments = ActivityExtensions.GetObjectsToExecute(context, backend, this).ToList();
            if (!attachments.Any())
            {
                throw new AutomationCanceledException();
            }

            var attachmentsToSign = attachments.Where(a => a.ActualFileSnapshot.Files.Any(f => f.Signatures.Any(s => s.PositionId == context.InitiatingPerson.MainPosition.Position)));
            foreach (var doc in attachmentsToSign)
            {
                await _digitalSigner.SignDocumentWithCertificateAsync(doc, person.Positions.Select(p => p.Position), SelectCertificate(), modifier);
            }
        }

        private static X509Certificate2Collection GetCertificatesCollection()
        {
            var store = new X509Store(StoreLocation.CurrentUser);
            store.Open(OpenFlags.OpenExistingOnly | OpenFlags.ReadOnly);
            var certificateCollection = store.Certificates;
            store.Close();

            certificateCollection = certificateCollection.Find(X509FindType.FindByTimeValid, DateTime.Now, true);
            certificateCollection = certificateCollection.Find(X509FindType.FindByKeyUsage, X509KeyUsageFlags.DigitalSignature, false);
            var withPrivate = new X509Certificate2Collection();
            foreach (X509Certificate2 cer in certificateCollection.Cast<X509Certificate2>().Where(cer => cer.HasPrivateKey))
                withPrivate.Add(cer);
            certificateCollection = withPrivate;

            return certificateCollection;
        }

        private X509Certificate2 SelectCertificate()
        {
            var certificateCollection = GetCertificatesCollection();

            if (certificateCollection.Count == 0)
                throw new AutomationException("Certificates for signing are not found or do not meet signing criteria");

            if (certificateCollection.Count == 1)
                return certificateCollection[0];

            var needChooseCertificate = "You need to choose the certificate for signing";
            certificateCollection = X509Certificate2UI.SelectFromCollection(
                certificateCollection,
                "Select certificate",
                needChooseCertificate,
                X509SelectionFlag.SingleSelection);

            if (certificateCollection.Count == 0)
            {
                throw new AutomationException(needChooseCertificate);
            }
            return certificateCollection[0];
        }
    }
}
